import callouts

@callouts.base
class PolyContext:
    @callouts.first
    def get_action_url(action: str):
        # TODO: log it too
        return 'ERROR: There is no implementation for PolyContext.get_action_url'


class Action:
    def __init__(self, name: str, url: str):
        self.name = name 
        self.url = url 

    def get_url_for(action: str) -> str:
        return PolyContext.get_action_url(action)


class Get(Action):
    def __init__(self, name: str, action: str = None):
        action = name if action is None else action
        super().__init__(name, Action.get_url_for(action))


class Home(Get):
    def __init__(self, action: str):
        super().__init__('home', action)


class Self(Get):
    def __init__(self, action: str):
        super().__init__('self', action)


class Poly():
    def __init__(self, title: str=None):
        self.title = title 
        self.actions = []
        self.data = []

    def set_title(self, title: str):
        self.title = title
        return self 

    def add_action(self, action: Action):
        self.actions.append(action)
        return self

    def add_data_item(self, obj: any): 
        self.data.append(obj)
        return self

    def to(self, formatter: any) -> dict: 
        return formatter.format(self)
        


import requests
from requests.models import Response
from .poly import Poly 
from .wstl import Wstl 
from .hal import Hal 

class RestClient: 
    def __init__(self):
        self.formatter = None
        self.format = 'json'
        self.poly = Poly()

    def get(self, url: str) -> Poly:
        headers = {
            'Accept': f'{Wstl.MIME_TYPE}, {Hal.MIME_TYPE}, application/json'
        }

        response = requests.get(url, headers=headers) 
        data = response.json()

        if 'wstl' in data.keys(): 
            self.format = 'wstl'
            self.formatter = Wstl
            self.poly = Wstl.parse(response.json())
        else: 
            self.poly = Poly()
            self.poly.add_data_item(data)
        
        return self.poly
    
    def do(self, action: str) -> Poly: 
        headers = {
            'Accept': self.formatter.MIME_TYPE
        }

        url = [a for a in self.poly.actions if a.name == action][0].url
        response = requests.get(url, headers=headers)
        self.poly = self.formatter.parse(response.json())

        return self.poly 

    def actions(self) -> list:
        return [a.name for a in self.poly.actions]

    def can_do(self, action: str) -> bool: 
        return 0 < len([a for a in self.poly.actions if a.name == action])